/*++

  COPYRIGHT        (C) NEC        Corporation 1997

  All rights reserved by NEC Corporation. This program must be
  used solely for the purpose for which it was furnished by NEC
  Corporation. No part of this program may be reproduced or
  disclosed to others, in any form, without the prior written
  permission of NEC Corporation.
  Use of copyright notice does not evidence publication of this
  program.

  Author	:  		T.Hirata(NEC Software Ltd.,)
  Module Name:		kmstruct.h

  Abstract:
	Kamui is Low-Level PowerVR Driver.
	'kmstruct.h' is kamui structure defines header.

  Environment:
	Microsoft Compiler.

  Notes:
	Kamui's structure.

  Revision History:
	97.08.19	Separeted from kmdefs.h

--*/
#if !defined(__kamui_struct_h)
#define __kamui_struct_h

#ifdef __cplusplus
extern "C" {
#endif 

/*++
 KMVERSIONINFO
 -------------


--*/
/*
 * Kamui Version Information structure
 */
typedef struct KMVERSIONINFO
{
	KMDWORD kmMajorVersion;  		/* ARC1 with IRIS 	= 1 */
									/* ARC1 with COSMOS	= 2	*/
									/* HOLLY			= 3	*/
	KMDWORD kmLocalVersion;
	KMDWORD kmFrameBufferSize; 		/* Total Size of TextureFrameBuffer */
} KMVERSIONINFO, *PKMVERSIONINFO;


/*++
color and palette


--*/
#if defined(_X86_)
#pragma pack(1)
#endif

/*
 * 32 bit packed color
 */
typedef union _tagKMPACKEDARGB
{
	KMDWORD	dwPacked;
	struct {
		KMBYTE	bBlue;
		KMBYTE	bGreen;
		KMBYTE	bRed;
		KMBYTE	bAlpha;
	}byte;
}KMPACKEDARGB,*PKMPACKEDARGB;

/*
 * PowerVR II OnChip Palette
 */

#define KM_PALETTE_ENTRY (1024)

typedef union _tagKMPALETTEDATA
{
	KMWORD	wPaletteData [KM_PALETTE_ENTRY];
	KMDWORD	dwPaletteData[KM_PALETTE_ENTRY];
}KMPALETTEDATA,*PKMPALETTEDATA;

#if defined(_X86_)
#pragma pack()
#endif

/*
 * FrameBuffer/TextureSurface
 */

typedef struct tagKMSURFACEDESC
{
	KMDWORD	SurfaceType;				/* 0:FrameBuffer    1:Texture	*/
	KMDWORD	BitDepth;					/* 16bpp...16   32bpp...32  etc	*/
 	KMDWORD	PixelFormat;				/* 1555,4444,etc				*/

	union{
		KMDWORD	USize;					/* USize 8-1024					*/
		KMDWORD	Width;					/* FB/Tex(Width)				*/
	}
#if defined(_STRICT_UNION_)
	u0
#endif
	;

	union{
		KMDWORD	VSize;					/* VSize 8-1024					*/
		KMDWORD	Height;					/* FB/Tex(Height)				*/
	}
#if defined(_STRICT_UNION_)
	u1
#endif
	;

	union{
		KMDWORD	dwTextureSize;			/* TextureSize					*/
		KMDWORD	dwFrameBufferSize;		/* FrameBufferSize				*/
	}
#if defined(_STRICT_UNION_)
	uSize
#endif
	;

	KMDWORD		fSurfaceFlags;			/* Surface Flags				*/
	PKMDWORD	pSurface;				/* Texture instance(Pointer on PVRII Core)	*/
	PKMDWORD	pVirtual;				/* Texture instance(Virtual  address on SH4)*/
	PKMDWORD	pPhysical;				/* Texture instance(physical address on SH4)*/

}KMSURFACEDESC;

typedef KMSURFACEDESC* PKMSURFACEDESC;

typedef PKMSURFACEDESC* PPKMSURFACEDESC;

/*
 * Callback functions
 */
typedef VOID (* PKMCALLBACKFUNC)( PKMVOID pArgs );


#ifdef __cplusplus
}
#endif 

#endif /*  __kamui_struct_h	*/

