//=============================================================================
//
// Copyright 1998 by Captivation Digital Laboratories, Inc.
// All Rights Reserved.
//
//=============================================================================
//
// "KRAD" -- Kamui Rendering/Animation Demonstration
//
// Note: These classes are adapted from utilities published
// in Graphics Gems IV, but they hardly resemble them anymore
//--------------------
// C++ Vector and Matrix Algebra routines
// Author: Jean-Francois DOUE
// Version 3.1 --- October 1993
//
//=============================================================================
//-----------------------------------------------------------------------------
// $Revision: 2 $
// $NoKeywords: $
//=============================================================================

#include "krad.h"
#include "kalgebra.h"
//=============================================================================
//=============================================================================	AGGRESSIVE MATH IMPLEMENTATION
//=============================================================================

float reciprocal_table[65536];
float squareroot_table[65536];


//=============================================================================
void InitializeAggressiveMath (void)
{
	volatile uint32 val = 0;
	for (uint32 i=0; i<65536 ;i++ )
	{
		reciprocal_table[i] = 1.0f / *((float*)(&val));
		squareroot_table[i] = (float) sqrt (*((float*)(&val)));
		val += 0x10000;
	}

}

//=============================================================================
//=============================================================================	GLOBAL:: output kvector, output kmatrix
//=============================================================================

ostream& operator << (ostream& s, kvector& v)
{ return s << "(" << v.n[VX] << ' ' << v.n[VY] <<' ' << v.n[VZ] << ")"; }

ostream& operator << (ostream& s, kmatrix& m)
{ return s << "[ " << m.m[VX] << " " << m.t[VX] << " ]\n"
		   << "[ " << m.m[VY] << " " << m.t[VY] << " ]\n"
		   << "[ " << m.m[VZ] << " " << m.t[VZ] << " ]"; }

//=============================================================================
//=============================================================================	kmatrix::inverse()
//=============================================================================
float	zed[8][4];

kmatrix& kmatrix::inverse()	    // Gauss-Jordan elimination with partial pivoting
{
	int		i,j;

	// init the gaussian field //
	for (i=0; i<3; i++)
	{
		for (j=0; j<3; j++)
			zed[i][j] =  m[j][i];
		zed[3][i] = t[i];
		zed[i][3] = 0.0f;
	}
	zed[3][3] = 1.0f;

	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			zed[i+4][j] = (i==j) ? 1.0f : 0.0f ;

	// run through the four columns and get play pivot games
	for (i=0; i<4; i++)
	{
		int		b = i;
		float	bf = zed[i][i];
		for (j=i+1; j<4; j++)
		{	// find highest pivot
			if (fabs(zed[i][j]) > bf)
			{
				b = j;
				bf= zed[i][j];
			}
		}

		// align pivot on a diagonal
		if (i != b)
		{
			for (j=0; j<8; j++)
			{
				float temp=zed[j][i];
						   zed[j][i]=zed[j][b];
					 	   			 zed[j][b]=temp;
			}
		}

		// make sure it's legal
		if (zed[i][i] == 0.0f)
			infoout << "kmatrix::inverse: singular matrix; can't invert\n";

		// scale whole row
		float r = 1.0f / zed[i][i];
		for (j=0; j<8; j++)
			zed[j][i] *= r;

		// eliminate elements in column
		for (j=0; j<4; j++)
		{
			if (i!=j)
			{
				float s = -zed[i][j];
				for (int k=0; k<8; k++)
					zed[k][j] += s * zed[k][i];
			}
		}
	}

	for (i=0; i<3; i++)
	{
		for (j=0; j<3; j++)
		{
			m[j][i] = zed[i+4][j];
		}
		t[i] = zed[7][i];
	}


//    kmatrix a(*this);		    // As a evolves from original mat into identity
//	kmatrix b(Identity);		// b evolves from identity into inverse(a)
//    int i, j, i1;
//
//    // Loop over first three cols of a from left to right, eliminating above and below diag
//    for (j=0; j<3; j++)
//	{   // Find largest pivot in column j among rows j..2, row three is always 0
//    	i1 = j;		    // Row with largest pivot candidate
//    	for (i=j+1; i<3; i++)
//    	{
//			if (fabs(a.m[i].n[j]) > fabs(a.m[i1].n[j]))
//				i1 = i;
//    	}
//
//    	// Swap rows i1 and j in a and b to put pivot on diagonal, don't forget the t column
//		if (i1 != j)
//		{
//	    	swap(a.m[i1], a.m[j]);	swap (a.t[i1], a.t[j]);
//	    	swap(b.m[i1], b.m[j]);	swap (b.t[i1], b.t[j]);
//		}
//
//    	// Scale row j to have a unit diagonal
//    	if (a.m[j].n[j]==0.0f)
//			infoout << "kmatrix::inverse: singular matrix; can't invert\n";
//
//		float r = 1.0f / a.m[j].n[j];
//    	a.m[j] *= r;	a.t[j] *= r;
//    	b.m[j] *= r;	b.t[j] *= r;
//
//    	// Eliminate off-diagonal elems in col j of a, doing identical ops to b
//    	for (i=0; i<3; i++)		// row 3 is trivial -- it's already zero
//    	{
//			if (i!=j)
//			{
//				b.t[i] -= a.m[i].n[j]*b.t[j];	// must do in reverse order, before scale value hit.
//				b.m[i] -= a.m[i].n[j]*b.m[j];
//				a.t[i] -= a.m[i].n[j]*a.t[j];
//				a.m[i] -= a.m[i].n[j]*a.m[j];
//			}
//    	}
//    }
//
//	// -- Do column 3: t -- //
//	// we can skip the pivoting since a[3,3] is guaranteed to be 1.0f
//	// we just need to eliminate the off-diagonals in the last column;
//    	// Eliminate off-diagonal elems in col j of a, doing identical ops to b
//   	for (i=0; i<3; i++)
//		b.t[i] -= a.t[i];	// the only one that matters
//
    return *this;
}

//=============================================================================	kmatrix::inverse()
kmatrix& kmatrix::inverse(const kmatrix& a)	    // Gauss-Jordan elimination with partial pivoting
{
	int		i,j;

	// init the gaussian field //
	for (i=0; i<3; i++)
	{
		for (j=0; j<3; j++)
			zed[i][j] =  a.m[j].n[i];
		zed[3][i] = a.t.n[i];
		zed[i][3] = 0.0f;
	}
	zed[3][3] = 1.0f;

	for (i=0; i<4; i++)
		for (j=0; j<4; j++)
			zed[i+4][j] = (i==j) ? 1.0f : 0.0f ;

	// run through the four columns and get play pivot games
	for (i=0; i<4; i++)
	{
		int		b = i;
		float	bf = zed[i][i];
		for (j=i+1; j<4; j++)
		{	// find highest pivot
			if (fabs(zed[i][j]) > bf)
			{
				b = j;
				bf= zed[i][j];
			}
		}

		// align pivot on a diagonal
		if (i != b)
		{
			for (j=0; j<8; j++)
			{
				float temp=zed[j][i];
						   zed[j][i]=zed[j][b];
					 	   			 zed[j][b]=temp;
			}
		}

		// make sure it's legal
		if (zed[i][i] == 0.0f)
			infoout << "kmatrix::inverse: singular matrix; can't invert\n";

		// scale whole row
		float r = 1.0f / zed[i][i];
		for (j=0; j<8; j++)
			zed[j][i] *= r;

		// eliminate elements in column
		for (j=0; j<4; j++)
		{
			if (i!=j)
			{
				float s = -zed[i][j];
				for (int k=0; k<8; k++)
					zed[k][j] += s * zed[k][i];
			}
		}
	}

	for (i=0; i<3; i++)
	{
		for (j=0; j<3; j++)
		{
			m[j][i] = zed[i+4][j];
		}
		t[i] = zed[7][i];
	}

    return *this;
}


//=============================================================================
//-----------------------------------------------------------------------------

//=============================================================================	END-OF-FILE