#--------------------------------------------------------------------------------------------#
#  CONFIDENTIAL AND PROPRIETARY:                                                             #
#       Copyright 1997, Sega of America.  All rights reserved.                               #
#       This sample source code is provided for demonstration purposes only.                 #
#       Use and distribution of Sega proprietary material is governed by                     #
#       developer licensing agreement.                                                       #
#--------------------------------------------------------------------------------------------#

#--------------------------------------------------------------------------------------------#
#  Name:     Katana.inc                                                                      #
#  Title:    Katana Makefile                                                                 #
#  Author:   Gary Lake                                                                       #
#  Created:  January 22, 1998                                                                #
#                                                                                            #
#  Version:  1.0                                                                             #
#  Platform: Katana | Windows CE | Kamui | Ninja                                             #
#                                                                                            #
#  Description:                                                                              #
#       Shared Makefile definitions required to build Katana projects.                       #
#                                                                                            #
#  History:                                                                                  #
#       1/22/98 - Program created (Gary Lake).                                               #
#--------------------------------------------------------------------------------------------#

.SILENT :

#--------------------------------------------------------------------------------------------#
# Directory path names                                                                       #
#--------------------------------------------------------------------------------------------#

# Includes

DEF_INC_PATH    = $(INCLUDE)  #c:\msdev\vc\include
KAMUI_INC_PATH  = ..\..\..\kamui\include
NINJA_INC_PATH  = ..\..\..\ninja\include

# Libraries

DEF_LIB_PATH    = $(LIB)      #c:\msdev\vc\lib
KAMUI_LIB_PATH  = ..\..\..\kamui\lib
NINJA_LIB_PATH  = ..\..\..\ninja\lib

# Output files

OUTDIR          = .\exe
OBJDIR          = .\obj


#--------------------------------------------------------------------------------------------#
# Standard definitions                                                                       #
#--------------------------------------------------------------------------------------------#

# Compiler and Linker

CPP             = cl.exe
LINK            = link.exe

# Default libraries

DEF_LIBS        = kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib\
                  advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib


#--------------------------------------------------------------------------------------------#
# Compiler switches                                                                          #
#--------------------------------------------------------------------------------------------#

# Debugging on/off

!ifdef DEBUG
CPP_DEBUG       = /D_DEBUG /MLd /Od /Zi /Gm
!else
CPP_DEBUG       = /DNDEBUG /ML /O2
!endif

# Windows console application yes/no

!ifdef CONSOLE
CPP_CONSOLE     = /D_CONSOLE
!else
CPP_CONSOLE     =
!endif

# Generate assembly output yes/no

!ifdef LIST
CPP_LIST        = /FAcs
!else
CPP_LIST        =
!endif


#--------------------------------------------------------------------------------------------#
# Compiler defines                                                                           #
#--------------------------------------------------------------------------------------------#

CPP_DEFS        = $(CPP_CONSOLE) /DWIN32 /D_WINDOWS /D_WIN32 /DWIN32_LEAN_AND_MEAN\
                  /D_X86_ /DIS_32


#--------------------------------------------------------------------------------------------#
# Compiler options                                                                           #
#--------------------------------------------------------------------------------------------#

CPP_OPTS        = /nologo /G5 /W3 /GX /FD $(CPP_LIST)\
                  /YX /Fp"$(OBJDIR)\$(PROJECT).pch"\
                  /Fo"$(OBJDIR)\\"\
                  /Fd"$(OBJDIR)\\"\
                  /I"$(DEF_INC_PATH)"\
                  /I"$(KAMUI_INC_PATH)"\
                  /I"$(NINJA_INC_PATH)"

CPP_FLAGS       = $(CPP_OPTS) $(CPP_DEFS) $(CPP_DEBUG)


#--------------------------------------------------------------------------------------------#
# Linker switches                                                                            #
#--------------------------------------------------------------------------------------------#

# Debugging on/off

!ifdef DEBUG
LINK_DEBUG      = /pdb:"$(OUTDIR)\$(PROJECT).pdb" /debug /pdbtype:sept /map
!else
LINK_DEBUG      = /pdb:none
!endif

# Windows console application yes/no

!ifdef CONSOLE
LINK_CONSOLE    = /subsystem:console
!else
LINK_CONSOLE    = /subsystem:windows
!endif


#--------------------------------------------------------------------------------------------#
# Linker options                                                                             #
#--------------------------------------------------------------------------------------------#

LINK_OPTS       = /nologo $(LINK_CONSOLE) /machine:I386 /incremental:no\
                  /libpath:"$(DEF_LIB_PATH)"\
                  /libpath:"$(KAMUI_LIB_PATH)"\
                  /libpath:"$(NINJA_LIB_PATH)"\
                  /out:"$(OUTDIR)\$(PROJECT).exe"

LINK_FLAGS      = $(DEF_LIBS) $(PROJECT_LIBS) $(LINK_OPTS) $(LINK_DEBUG)


#--------------------------------------------------------------------------------------------#
# Build rules                                                                                #
#--------------------------------------------------------------------------------------------#

# Build C/C++ files

.c{$(OBJDIR)}.obj::
    $(CPP) @<<
    $(CPP_FLAGS) /c $<
<<

.cpp{$(OBJDIR)}.obj::
    $(CPP) @<<
    $(CPP_FLAGS) /c $<
<<

# Link object files into exe

"$(OUTDIR)\$(PROJECT).exe" : "$(OUTDIR)" "$(OBJDIR)" $(PROJECT_OBJS)
    $(LINK) @<<
    $(LINK_FLAGS) $(PROJECT_OBJS)
<<


#--------------------------------------------------------------------------------------------#
# Makefile directives                                                                        #
#--------------------------------------------------------------------------------------------#

# Make all

ALL :
    "$(OUTDIR)\$(PROJECT).exe"

# Clean temporary files

CLEAN :
    -@erase "$(OBJDIR)\*.obj"
    -@erase "$(OBJDIR)\*.idb"
    -@erase "$(OBJDIR)\*.pdb"
    -@erase "$(OBJDIR)\*.pch"
    -@erase "$(OUTDIR)\*.map"
    -@erase "$(OUTDIR)\*.pdb"
    -@erase "$(OUTDIR)\*.exe"

# Create directories

"$(OUTDIR)" :
    if not exist "$(OUTDIR)/$(NULL)" mkdir "$(OUTDIR)"

"$(OBJDIR)" :
    if not exist "$(OBJDIR)/$(NULL)" mkdir "$(OBJDIR)"