/*
	Ninja Library
	
	COPYRIGHT (C) SEGA ENTERSRISES,LTD.
*/

#ifndef _NINJA_API_H_
#define _NINJA_API_H_


/*--------------------------------------*/
/*      Prototype                       */
/*--------------------------------------*/

/************************/
/*	System				*/
/************************/
void	njInitSystem( Int, Int, Int );
void	njInitVertexBuffer( Uint32 op, Uint32 om, Uint32 tp, Uint32 tm );
void	njWaitVSync( void );
void	njSetVSyncFunction( void (*func)() );
Uint32	njVersion( void );
void	njExitSystem( void );
void	njSetBackColor( Uint32 col1, Uint32 col2, Uint32 col3 );
void	njTextureFilterMode( Int mode );
void	njTextureClampMode( Int mode );
void	njTextureFlipMode( Int mode );
void	njMipmapAdjust( Int level );
void	njSuperSampleMode( Int mode );
/*  */
void    njPolygonCullingSize( Float size );
void    njPolygonCullingMode( Int mode );
void    njColorBlendingMode( Int target, Int mode );
void    njSpecularMode( Int mode );
void    njAlphaMode( Int mode );
void    njIgnoreTextureAlphaMode( Int mode );
void    njTextureShadingMode( Int mode );

/************************/
/*	Draw				*/
/************************/
void    njDrawPolygon( NJS_POLYGON_VTX *polygon, Int count, Int trans );
void    njDrawTexture( NJS_TEXTURE_VTX *polygon, Int count, Int tex, Int flag );

/************************/
/*	Model				*/
/************************/
void	njInit3D( NJS_VERTEX_BUF *vbuf, Int vn );
void	njDrawModel( NJS_MODEL *model );
void	njDrawObject( NJS_OBJECT *obj );
void	njFastDrawModel( NJS_MODEL *model );
void	njFastDrawObject( NJS_OBJECT *obj );
void	njSimpleDrawModel( NJS_MODEL *model );
void	njSimpleDrawObject( NJS_OBJECT *obj );
void	njShadowModel( NJS_MODEL *model, Uint32 Brate, Uint32 Orate );
void	njShadowObject( NJS_OBJECT *obj, Uint32 Brate, Uint32 Orate );
void	njControl3D( Uint32 flag );
void	njSetConstantAttr( Uint32 and_attr, Uint32 or_attr );
void	njSetConstantMaterial( NJS_ARGB *mat );
void	njSetDepthQueue( Float dnear, Float dfar );
void	njSetModObject( NJS_OBJECT *obj );
void	njSetModModel( NJS_MODEL *obj );
void	njSetSimpleLight( NJS_VECTOR *lit );

/************************/
/*	Motion				*/
/************************/
void	njAction( NJS_ACTION *action, Float frame );
void	njDrawMotion( NJS_OBJECT *object, NJS_MOTION *motion, Float frame );
void	njFastAction( NJS_ACTION *action, Float frame );
void	njFastDrawMotion( NJS_OBJECT *object, NJS_MOTION *motion, Float frame );
void	njSimpleAction( NJS_ACTION *action, Float frame );
void	njSimpleDrawMotion( NJS_OBJECT *object, NJS_MOTION *motion, Float frame );
void	njSetMotionCallback( void (*func)(NJS_OBJECT *obj) );

void	njActionLink( NJS_ACTION_LINK *actionlink, Float frame );
void	njDrawMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, Float frame );
void	njFastActionLink( NJS_ACTION_LINK *actionlink, Float frame );
void	njFastDrawMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, Float frame );
void	njSimpleActionLink( NJS_ACTION_LINK *actionlink, Float frame );
void	njSimpleDrawMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, Float frame );

void	njInitShape(Float *buf);
void	njDrawShapeMotion( NJS_OBJECT *object, NJS_MOTION *motion, NJS_MOTION *shape, Float frame );
void	njDrawShapeMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, NJS_MOTION_LINK *shapelink, Float frame );
void	njFastDrawShapeMotion( NJS_OBJECT *object, NJS_MOTION *motion, NJS_MOTION *shape, Float frame );
void	njFastDrawShapeMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, NJS_MOTION_LINK *shapelink, Float frame );
void	njSimpleDrawShapeMotion( NJS_OBJECT *object, NJS_MOTION *motion, NJS_MOTION *shape, Float frame );
void	njSimpleDrawShapeMotionLink( NJS_OBJECT *object, NJS_MOTION_LINK *motionlink, NJS_MOTION_LINK *shapelink, Float frame );

void	njMotionTransform( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njMotionTransformZXY( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njMotionTranslate( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njMotionRotateXYZ( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njMotionRotateZXY( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njMotionScale( NJS_MATRIX *m, NJS_OBJECT *obj, Int spline_flag );
void	njGetMotionTranslate( Float *pos, Int spline_flag );
void	njGetMotionRotate( Angle *ang, Int spline_flag );
void	njGetMotionScale( Float *scl, Int spline_flag );
void	njSetNextMotionNode( void );
void	njSetCurrentMotion(NJS_MOTION *motion, Float frame );

/************************/
/*	Memory				*/
/************************/
void	njMemCopy( void *dst, void *src, Sint32 num );
void	njMemCopy2( void *dst, void *src, Sint32 num );
void	njMemCopy4( void *dst, void *src, Sint32 num );

/************************/
/*	Draw2D				*/
/************************/

void njDrawPoint2D(NJS_POINT2COL *p, Int n, Float pri, Uint32 attr);
void njDrawLine2D(NJS_POINT2COL *p, Int n, Float pri, Uint32 attr);
void njDrawTriangle2D(NJS_POINT2COL *p, Int n, Float pri, Uint32 attr);
void njDrawPolygon2D(NJS_POINT2COL *p, Int n, Float pri, Uint32 attr);
void njDrawCircle2D(NJS_POINT2COL *p, Int n, Float pri, Uint32 attr);

/************************/
/*	Math				*/
/************************/

#define njAbs(n) ((Float)fabs((Double)(n)))
#define njArcCos(n) (NJM_RAD_ANG(acos((Double)(n))))
#define njArcCosec(n) (NJM_RAD_ANG(asin(1.0/(Double)(n))))
#define njArcCot(n) (NJM_RAD_ANG(atan(1.0/(Double)(n))))
#define njArcSec(n) (NJM_RAD_ANG(acos(1.0/(Double)(n))))
#define njArcSin(n) (NJM_RAD_ANG(asin((Double)(n))))
#define njArcTan(n) (NJM_RAD_ANG(atan((Double)(n))))
#define njArcTan2(y,x) (NJM_RAD_ANG(atan2((Double)(y),(Double)(x))))
#define njCeil(n) ((Float)ceil((Double)(n)))
#define njCosech(n) ((Float)(1.0 / sinh((Double)NJM_ANG_RAD(n))))
#define njCosh(n) ((Float)cosh((Double)NJM_ANG_RAD(n)))
#define njCoth(n) ((Float)(1.0 / tanh((Double)NJM_ANG_RAD(n))))
#define njFloor(n) ((Float)floor((Double)(n)))
#define njInvertSqrt(n) (Float)(1.0 / njSqrt(n))
#define njLog(n) ((Float)log((Double)(n)))
#define njLog10(n) ((Float)log10((Double)(n)))
#define njLog2(n) (njLog(n) / njLog(2.f))
#define njPow(n1,n2) ((Float)pow((Double)(n1), (Double)(n2)))
#define njSech(n) ((Float)(1.0 / cosh((Double)NJM_ANG_RAD(n))))
#define njSinh(n) ((Float)sinh((Double)NJM_ANG_RAD(n)))
#define njTanh(n) ((Float)tanh((Double)NJM_ANG_RAD(n)))
#define njHypot(x,y) njSqrt(x * x + y * y)
#define njExp(x) (Float)exp(x)
#define njRandomSeed(n)	srand(n)

Float njCos(Angle);
Float njCosec(Angle);
Float njCot(Angle);
Float njFraction(Float);
Float njRandom();
Float njRoundOff(Float);
Float njRoundUp(Float);
Float njSec(Angle);
Float njSin(Angle);
Float njSqrt(Float);
Float njTan(Angle);
/* Float njSort(Float); */


/************************/
/*	Matrix				*/
/************************/

void	njInitMatrix(NJS_MATRIX *m, Sint32 n, Int flag);

/*  */
void    njClearMatrix();

Bool	njPushMatrix(NJS_MATRIX *m);
Bool	njPopMatrix(Uint32 n);
void	njUnitMatrix(NJS_MATRIX *m);
void	njSetMatrix(NJS_MATRIX *md, NJS_MATRIX *ms);
void	njGetMatrix(NJS_MATRIX *m);
void	njAddMatrix(NJS_MATRIX *md, NJS_MATRIX *ms);
void	njSubMatrix(NJS_MATRIX *md, NJS_MATRIX *ms);
void	njMultiMatrix(NJS_MATRIX *md, NJS_MATRIX *ms);
void	njTranslate(NJS_MATRIX *m, Float x, Float y, Float z);
void	njTranslateV(NJS_MATRIX *m, NJS_VECTOR *v);
void	njRotateX(NJS_MATRIX *m, Angle ang);
void	njRotateY(NJS_MATRIX *m, Angle ang);
void	njRotateZ(NJS_MATRIX *m, Angle ang);
void	njRotateXYZ(NJS_MATRIX *m, Angle angx, Angle angy, Angle angz);
void	njRotateZXY(NJS_MATRIX *m, Angle angx, Angle angy, Angle angz);
void	njRotate(NJS_MATRIX *m, NJS_VECTOR *v, Angle ang);
void	njScale(NJS_MATRIX *m, Float sx, Float sy, Float sz);
void	njScaleV(NJS_MATRIX *m, NJS_VECTOR *v);
Bool	njInvertMatrix(NJS_MATRIX *m);
void	njTransposeMatrix(NJS_MATRIX *m);
Float	njDetMatrix(NJS_MATRIX *m);
void	njResMatrix(NJS_MATRIX *m, Angle *ang,NJS_VECTOR *scale,NJS_VECTOR *trans);
void	njProject(NJS_MATRIX *m,NJS_PLANE *pl);
void    njProject2(NJS_MATRIX *m, NJS_PLANE *pl, NJS_POINT3 *v, NJS_POINT3*p);
void	njMirror(NJS_MATRIX *m,NJS_PLANE *pl);

void	njCalcPoint(NJS_MATRIX *m, NJS_POINT3 *ps, NJS_POINT3 *pd);
void	njAddVector(NJS_VECTOR *vd, NJS_VECTOR *vs);
void	njSubVector(NJS_VECTOR *vd, NJS_VECTOR *vs);
void	njCalcVector(NJS_MATRIX *m, NJS_VECTOR *vs, NJS_VECTOR *vd);
Float	njUnitVector(NJS_VECTOR *v);
Float	njScalor(NJS_VECTOR *v);
Float	njScalor2(NJS_VECTOR *v);
void	njProjectScreen(NJS_MATRIX *m, NJS_POINT3 *p3, NJS_POINT2 *p2);
Float	njOuterProduct(NJS_VECTOR *v1, NJS_VECTOR *v2, NJS_VECTOR *ov);
Float	njInnerProduct(NJS_VECTOR *v1, NJS_VECTOR *v2);
void    njGetPlaneNormal(NJS_POINT3 *p, NJS_VECTOR *v);
void    njGetPlaneNormal2(NJS_POINT3 *p0, NJS_POINT3 *p1, NJS_POINT3 *p2, NJS_VECTOR *v);


/************************/
/*	Collision   		*/
/************************/

Bool	njIsParalellL2L(NJS_LINE *l1, NJS_LINE *l2);
Bool	njIsParalellL2PL(NJS_LINE *l, NJS_PLANE *pl);
Bool	njIsParalellPL2PL(NJS_PLANE *pl1, NJS_PLANE *pl2);
Float	njDistanceP2P(NJS_POINT3 *p1, NJS_POINT3 *p2);
Float	njDistanceP2L(NJS_POINT3 *p, NJS_LINE *l, NJS_POINT3 *cp);
Float	njDistanceP2PL(NJS_POINT3 *p, NJS_PLANE *pl, NJS_POINT3 *cp);
Float	njDistanceL2L(NJS_LINE *l1, NJS_LINE *l2, NJS_POINT3 *cp1, NJS_POINT3 *cp2);
Float	njDistanceL2PL(NJS_LINE *l, NJS_PLANE *pl, NJS_POINT3 *cp);
Float	njDistancePL2PL(NJS_PLANE *pl1, NJS_PLANE *pl2, NJS_LINE *l);
/*  */
Int     njCollisionCheckBB(NJS_BOX *h1, NJS_BOX *h2);
Int     njCollisionCheckSS(NJS_SPHERE *sphere1, NJS_SPHERE *sphere2);
Int     njCollisionCheckCC(NJS_CAPSULE *h1, NJS_CAPSULE *h2);
Int     njCollisionCheckSC(NJS_SPHERE *sphere, NJS_CAPSULE *capsule);
Int     njCollisionCheckBS(NJS_BOX *box, NJS_SPHERE *sphere);
Int     njCollisionCheckBC(NJS_BOX *box, NJS_CAPSULE *capsule);


/************************/
/*	View				*/
/************************/

void    njSetScreen(NJS_SCREEN *s);
void    njSetScreenDist(Float dist);
void	njSetPerspective(Angle ang);
void    njSetAspect(Float ax, Float ay);
void    njInitView(NJS_VIEW *v);
void    njSetView(NJS_VIEW *v);
/*  */
void    njUnitViewMatrix(NJS_VIEW *v);
void    njMultiViewMatrix(NJS_VIEW *v, NJS_MATRIX *m);
void    njRotateViewX(NJS_VIEW *v, Angle ang);
void    njRotateViewY(NJS_VIEW *v, Angle ang);
void    njRotateViewZ(NJS_VIEW *v, Angle ang);
void    njRotateViewXYZ(NJS_VIEW *v, Angle ax, Angle ay, Angle az);
void    njTranslateView(NJS_VIEW *v, Float x, Float y, Float z);
void    njTranslateViewV(NJS_VIEW *v, NJS_POINT3 *p);
/* */
void    njRotateViewXRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewYRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewZRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewXYZRelative(NJS_VIEW *v, Angle ax, Angle ay, Angle az);
void    njTranslateViewRelative(NJS_VIEW *v, Float x, Float y, Float z);
void    njTranslateViewVRelative(NJS_VIEW *v, NJS_POINT3 *p);
void    njForwardViewRelative(NJS_VIEW *v, Float x);
void    njRotateViewXAbsolute(NJS_VIEW *v, Angle ang);
void    njRotateViewYAbsolute(NJS_VIEW *v, Angle ang);
void    njRotateViewZAbsolute(NJS_VIEW *v, Angle ang);
void    njRotateViewXYZAbsolute(NJS_VIEW *v, Angle ax, Angle ay, Angle az);
void    njTranslateViewAbsolute(NJS_VIEW *v, Float x, Float y, Float z);
void    njTranslateViewVAbsolute(NJS_VIEW *v, NJS_POINT3 *p);
void    njForwardViewAbsolute(NJS_VIEW *v, Float x);
void    njSetBaseView(NJS_VIEW *v);
void    njReturn2BaseView(NJS_VIEW *v);
void	njLookAtView(NJS_VIEW *v, Float x, Float y, Float z);
void	njLookAtViewV(NJS_VIEW *v, NJS_POINT3 *p);
void    njClip2D(NJS_POINT2 *v);
void    njRender();
void    njClipZ(Float n, Float f);
int     njCalcScreen(NJS_POINT3 *p, Float *sx, Float *sy);
int     njNearClipping(NJS_POINT3COL *p, Int n, NJS_POINT3COL *cp, Uint32 attr);
void    njRotateViewPosXRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewPosYRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewPosZRelative(NJS_VIEW *v, Angle ang);
void    njRotateViewPosXAbsolute(NJS_VIEW *v, Angle ang);
void    njRotateViewPosYAbsolute(NJS_VIEW *v, Angle ang);
void    njRotateViewPosZAbsolute(NJS_VIEW *v, Angle ang);
void    njUnitViewVector(NJS_VIEW *v);

/************************/
/*	Graphics3D			*/
/************************/

void    njDrawPoint3D(NJS_POINT3COL *p, Int n, Uint32 attr);
void    njDrawLine3D(NJS_POINT3COL *p, Int n, Uint32 attr);
void    njDrawTriangle3D(NJS_POINT3COL *p, Int n, Uint32 attr);
void    njDrawPolygon3D(NJS_POINT3COL *p, Int n, Uint32 attr);

/************************/
/*	 Light Funcion		*/
/************************/
void	njCreateLight(NJS_LIGHT_PTR, Int);
void	njDeleteLight(NJS_LIGHT_PTR);
void	njLightOn(NJS_LIGHT_PTR);
void	njLightOff(NJS_LIGHT_PTR);
void	njSetLight(NJS_LIGHT_PTR);

void	njSetLightLocation(NJS_LIGHT_PTR, Float, Float, Float);
void	njSetLightDirection(NJS_LIGHT_PTR, Float, Float, Float);
void	njSetLightColor(NJS_LIGHT_PTR, Float, Float, Float);
void	njSetLightAlpha(NJS_LIGHT_PTR, Float);
void	njSetLightIntensity(NJS_LIGHT_PTR, Float, Float, Float);
void	njSetLightAngle(NJS_LIGHT_PTR, Angle, Angle);
void	njSetLightRange(NJS_LIGHT_PTR, Float, Float);
void	njSetUserLight(NJS_LIGHT_PTR, NJF_LIGHT_FUNC);

void	njMultiLightMatrix(NJS_LIGHT_PTR, NJS_MATRIX*);
void	njRotateLightXYZ(NJS_LIGHT_PTR, Angle, Angle, Angle);
void	njRotateLightX(NJS_LIGHT_PTR, Angle);
void	njRotateLightY(NJS_LIGHT_PTR, Angle);
void	njRotateLightZ(NJS_LIGHT_PTR, Angle);
void	njTranslateLightV(NJS_LIGHT_PTR, NJS_VECTOR*);
void	njTranslateLight(NJS_LIGHT_PTR, Float, Float, Float);
void	njUnitLightMatrix(NJS_LIGHT_PTR);

/************************/
/*	 Texture Funcion	*/
/************************/
void njInitTexture(NJS_TEXMEMLIST *addr,Uint32 n);
void njExitTexture(void);
void njInitCacheTexture(NJS_TEXMEMLIST *addr,Uint32 n);
Sint32 njLoadTexture(NJS_TEXLIST *texlist);
Sint32 njLoadTextureNum(Uint32 n);
Sint32 njSetTexture(NJS_TEXLIST *texlist);
Sint32 njSetTextureNum(Uint32 n);
Sint32 njSetTextureNumG(Uint32 globalIndex);
Sint32 njGetTextureNumG(void);
void njReleaseTextureAll(void);
Sint32 njReleaseTexture(NJS_TEXLIST *texlist);
Sint32 njReleaseTextureNum(Uint32 n);
Sint32 njReleaseTextureNumG(Uint32 globalIndex);
void njReleaseCacheTextureAll(void);
Sint32 njReleaseCacheTextureNum(Uint32 n);
Sint32 njReleaseCacheTextureNumG(Uint32 globalIndex);
Uint32 njCalcTexture(Uint32 flag);
Sint32 njLoadCacheTexture(NJS_TEXLIST *texlist);
Sint32 njLoadCacheTextureNum(Uint32 n);
Sint32 njLoadCacheTextureNumG(Uint32 globalIndex);
void njSetTexturePath(Uint8 *path);
void njSetTextureInfo(NJS_TEXINFO *info,Uint16 *tex,Sint32 Type,Sint32 nWidth,Sint32 nHeight);
void njSetTextureName(NJS_TEXNAME *texname,void *addr,Uint32 globalIndex,Uint32 attr);
Sint32 njReLoadTextureNum(Uint32 n,void *texaddr,Uint32 attr,Uint32 lod);
Sint32 njReLoadTextureNumG(Uint32 globalIndex,void *texaddr,Uint32 attr,Uint32 lod);
void njSetRenderWidth(Uint32 nWidth);
void njFrameBufferBmp(Uint8 *filename);

/************************/
/*	 Scroll Funcion		*/
/************************/
void njDrawScroll(NJS_SCROLL *scl);


/************************/
/*	 Debug Font Funcion	*/
/************************/
void njPrintSize(Uint16 size);
void njPrintColor(Uint32 c);
void njPrintC(Int loc,Uint8* s);
void njPrintD(Int loc,Int val,Int digit);
void njPrintB(Int loc,Uint32 val,Int digit);
void njPrintH(Int loc,Uint32 val,Int digit);
void njPrintF(Int loc,Float val,Int digit);

/************************/
/*	 Sprite				*/
/************************/
void	njDrawSprite2D( NJS_SPRITE *sp, Int n, Float pri, Uint32 attr );
void	njDrawSprite3D( NJS_SPRITE *sp, Int n, Uint32 attr );

/************************/
/*	 Fog				*/
/************************/
void njFogEnable();
void njFogDisable();
void njSetFogDensity(Uint32 density);
void njSetFogColor(Uint32 c);
void njSetFogTable(NJS_FOG_TABLE fog);
void njGenerateFogTable(NJS_FOG_TABLE fog);
void njGenerateFogTable2(NJS_FOG_TABLE fog, Float f);
void njGenerateFogTable3(NJS_FOG_TABLE fog, Float n, Float f);

/************************/
/*	 Fade				*/
/************************/
void njFadeEnable();
void njFadeDisable();
void njSetFadeColor(Uint32 c);
void njExecuteFade(Float f);

/************************/
/*	 Chromakey   		*/
/************************/
/* void njChromakeyEnable(); */
/* void njChromakeyDisable(); */
/* void njSetChromakeyColor(Uint32 c); */

#endif /* _NINJA_API_H_ */
